/*******************************************************************************

 BURGLAR_SMS_ALERT

 Alerta de disparo de alarme via SMS

 C/ Pic 12f675 e mdulo SIM800L

 Autor: Cludio Lrios

 Data: 04/04/2019

 Digite abaixo o Nmero do Telefone Celular que receber o SMS de alerta.

 Depois recompile este programa para obter o arquivo.hex para gravar o PIC.

 Blog Picsource

 Uso didtico

*******************************************************************************/

#include "burglar_sms_alert.h" //arquivo de cabealho
#use fast_io(a)


//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

#rom 0x2100 = {"XXXXXXXXXXXXX"}//COLOQUE AQUI O NUMERO DE SEU TELEFONE (DDD+NUM)
// exemplo    {"032988776110"} //obs. numero de celular qualquer para exemplo

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

//bytes
#byte trisio = 0x85 //sentido dos pinos
#byte gpio   = 0x05 //gpio
#byte tmr0   = 0x01

//bit
#bit t0if = 0xb.2

//defines
#define sw_save  PIN_A3//0x5.3  - pino 4
#define in_disp  PIN_A0//0x5.0  - pino 7
#define CEL_NUMBER_SIZE 18
#define NUM_MINIMO 7  //quantidade minima de caracteres numricos no cel_buffer
#define TEMPO_POS_DISPARO 5//aps disparo do alarme, envia sms aps este tempo
//variveis globais
unsigned int1 flagt1=0,flag_cel=0;
char r;
unsigned int8 cel_index=0;       //ndice de n. celular
unsigned int8 a;
char cel_buffer[CEL_NUMBER_SIZE];//armazena numero da ltima ligao de celular
unsigned int32 time_out=0;

/******************************************************************************
    Rotina de Delay de 1 Segundo
 ******************************************************************************/

void delay_1s(){
  delay_ms(1000);//delay 1 segundo
}
/******************************************************************************
    Rotina de DElay para 'N' segundos
 ******************************************************************************/
void delay_s(unsigned int8 valor){
unsigned int8 a;
  for(a=0;a<valor;a++){
   delay_1s();
  }

}
/******************************************************************************
    Rotina de Escrita na EEprom Interna do Pic
 ******************************************************************************/
void write_eep(unsigned int8 address,valor){
  write_eeprom(address,valor);
}
/******************************************************************************
    Rotina de Leitura na EEprom Interna do Pic
 ******************************************************************************/
unsigned int8 read_eep(unsigned address){
  return read_eeprom(address);
}
/******************************************************************************
    Rotina de Escrita do Nmero de Celular na EEprom Interna do Pic
 ******************************************************************************/

void salva_numero_EEp(){//salva numero do celular que ligou na eeprom
 unsigned int8 i;
  for(i=0;i<cel_index;i++){
   write_eep(i,cel_buffer[i]);
  }
  write_eep(cel_index,0); //coloca o ponto de 'parada' do novo nmero
}
 /******************************************************************************
    Rotina de Envio de SMS p/ Confirmar que o Nmero do Celular foi Salvo
    (o SMS ir ser enviado para o mesmo nmero salvo)
 ******************************************************************************/
void env_sms_confirmacao(){
  char val;
 /******************************************************************************
    Informa ao mdulo qual celular ser enviado a confirmao do comando
 ******************************************************************************/
       printf("AT+CMGF=1\r\n");//modo texto
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       //comando para enviar numero do celular
       printf("AT+CMGS=\"");//envia aspas iniciais com cdigo de barra invertida \"
       for(a=0;a<CEL_NUMBER_SIZE;a++){
          val=read_eep(a);
          if(val>'9'||val<'0')break;//fim dos numeros
          printf("%c", val);//envia dgitos do numero do celular salvo na EEprom
       }
       printf("\"\r\n");//envia aspas finais e 'enter'

       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       printf("BURGLAR_SMS_ALERT-> NUMERO DE CELULAR SALVO");//mensagem
       printf("\x1A"); //envia control z
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       cel_index=0;//zera indice de tamanho do numero do ltimo celular

}//env_sms_confirmacao()

 /******************************************************************************
    Rotina de Envio de Mensagem de Alerta por Disparo de Alarme
 ******************************************************************************/
void env_sms_alerta(){
  char val;

       printf("AT+CMGF=1\r\n");//modo texto
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       //comando para enviar numero do celular
       printf("AT+CMGS=\"");//envia aspas iniciais com cdigo de barra invertida \"
       for(a=0;a<CEL_NUMBER_SIZE;a++){
          val=read_eep(a);
          if(val>'9'||val<'0')break;//fim dos numeros
          printf("%c", val);//envia dgitos do numero do celular salvo na EEprom
       }
       printf("\"\r\n");//envia aspas finais e 'enter'

       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       printf("BURGLAR_SMS_ALERT-> ATENCAO: ALARME FOI DISPARADO!");//mensagem
       printf("\x1A"); //envia control z
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando

}
/******************************************************************************
     Rotina Main - Configuraes dos Registradores
*******************************************************************************/


void main() {
#define TEMPO_MAX 2000000

   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   trisio= 0b111101;
   delay_s(20);//delay de 20 segundos para o mdulo SIM800L se estabilizar
   printf("AT\r\n");//envia comando apenas para sincronizar baudrate
   delay_1s();//aguarde
   printf("ATE0\r\n");//envia comando p/ no ecoar comandos
   delay_1s();//aguarde
   printf("AT+CMGF=1\r\n");//envia comando para modo texto
   delay_1s();//aguarde
   printf("AT+CLIP=1\r\n");//envia comando de identifio de chamadas
   delay_1s();//aguarde

/******************************************************************************
     Loop Continuo
*******************************************************************************/

   for(;;){
//teste



/*******************************************************************************
   Rotina para salvamento de numero do celular.
   Atravs do comando de identifio de chamadas (at+clip=1) ser visto o
   nmero que liga a cada "ring".
*******************************************************************************/
     if(!input(sw_save))flagt1=1;
     while(flagt1){//teste interruptor 'salva_n"
       if(kbhit()){//testa se iniciou uma transmisso
         r=getc();//sim, pegando o caracter
         time_out=0;
         if((r>='0')&&(r<='9')){//captura apenas numeros de 0 a 9
              if(cel_index<CEL_NUMBER_SIZE)cel_buffer[cel_index++]=r;
           }
           else{//se no  numero recomea buffer
             if(cel_index<NUM_MINIMO){//se foi sequencia curta de numeros
               cel_index=0;//recomea o ponteiro do buffer
             }
             else{//finaliza a captura do nmero se sequencia maior que n. mnimo
               if(r=='\"'){
                flag_cel=1;  //e recebeu aspas (temos o nmero completo)
                flagt1=0;
                salva_numero_EEp();//salva na eeprom
                printf("AT+ATH\r\n");//no aceita a chamada (fone no gancho)
                delay_1s();//aguarda receber 'ok' antes de enviar outro comando
                env_sms_confirmacao();
                break;
               }
             }
           }//else
       }//if(kbhit()...
       else{
       time_out++;
       if(time_out>TEMPO_MAX){
           time_out=0;
           flagt1=0;//sa do lao
       }//if(time_out>TEMPO_MAX...
      }//else
     }//while(!sw_save||flagt1)...

/******************************************************************************
       Rotina de verificao de Disparo e Envio de SMS
*******************************************************************************/

    if(input(in_disp)){
     //se for disparado pega da eeprom o nmero e envia SMS
       delay_s(TEMPO_POS_DISPARO);//p/ no acionar quando liga e desliga alarme
        if(input(in_disp)){
         env_sms_alerta();
         while(input(in_disp))delay_s(1);//aguarda por desligamento do alarme
       }// if(!in_disp)
    }// if(!in_disp)


   }//for(;;)

}//main()
//=============================fim==============================================
